class Logger():
    _logger = None

    def __init__(self, logger = None):
        self._logger = logger

    def debug(self, message):
        if self._logger is None:
            print("DEBUG: " + message)
            return

        self._logger.debug(message)

    def verbose(self, message):
        if self._logger is None:
            print("VERBOSE: " + message)
            return

        self._logger.verbose(message)

    def info(self, message):
        if self._logger is None:
            print("INFO: " + message)
            return

        self._logger.info(message)

    def warning(self, message):
        if self._logger is None:
            print("WARNING: " + message)
            return

        self._logger.warning(message)

    def error(self, message):
        if self._logger is None:
            print("ERROR: " + message)
            return

        self._logger.error(message)


import json
import subprocess

from pyexecutor import Logger


class Commander():
    _output = ''
    _error = ''
    _returncode = 0
    _logger = None

    def __init__(self, logger=None):
        self._set_logger(logger)

    """
    Run command with sub process
    """
    def run(self, executor, args='', supress_error=False):
        if supress_error:
            self._logger.warning('Supress error is True, will not throw any exceptions even if any error occured when running command!')

        try:
            command = [executor] + args.strip().split(' ')
            self._logger.debug('Running command %s.' % (' '.join(command)))

            result = subprocess.run(command, capture_output=True)
            self._output = result.stdout
            self._error = result.stderr

            if result.returncode != 0:
                raise CommanderException(self.error())

            self._returncode = 0

            self._logger.info('Run command sucessfully!')
            return self
        except Exception as e:
            self._logger.error('%s' % (e))

            self._returncode = 1

            if not supress_error:
                raise e

            self._logger.warning('The command error is supressed due to supress_error is True!')

            return self

    """
    Get output message
    """
    def output(self):
        return self._output.decode("utf8").strip()

    """
    Get output message in JSON format
    """
    def json(self):
        try:
            return json.loads(self.output())
        except Exception as e:
            self._logger.error('Cannot convert command output to JSON object, %s' % (e))
            raise CommanderException('Invalid JSON string "%s"' % (self._output))

    """
    Get error message
    """
    def error(self):
        return self._error.decode("utf8").strip()

    """
    Return code is 0
    """
    def ok(self):
        return self._returncode == 0

    """
    Return code is not 0
    """
    def fail(self):
        return self._returncode != 0

    """
    Set the command logger
    """
    def _set_logger(self, logger):
        self._logger = Logger(logger)

class CommanderException(Exception): ...


import platform

from pyexecutor import Commander
from pyexecutor import Logger


class Executor():

    _logger = None
    _commander = None
    _executor = None
    _trailer = ''

    def __init__(self, executable, logger=None):
        self._commander = Commander(logger=logger)
        self._set_logger(logger=logger)
        self._set_executor(executable)

    """
    Choose available executor
    """
    def _set_executor(self, executor):
        system = platform.system()

        if system == 'Linux' or system == 'Darwin':
            self._search_linux(executor)
        elif system == 'Windows':
            self._search_windows(executor)
        else:
            raise ExecutorException('Unsupported OS type %s!' % (system))

        if self._executor is None:
            raise ExecutorException('Executor %s not found' % (executor))

        self._logger.info('Choose "%s" as executor' % (self._executor))

    """
    Search available executors in Linux system
    """
    def _search_linux(self, executor):
        result = self._commander.run("which", executor, True)

        executors = result.output().replace("\n", "&PYEXECUTOR&").replace("\r", "&PYEXECUTOR&").split("&PYEXECUTOR&")

        self._logger.debug("Available executors found: %s" % executors)

        if len(executors) > 0:
            self._executor = executors[0]

    """
    Search available executors in Windows system
    """
    def _search_windows(self, executor):
        result = self._commander.run("where", executor, True)

        executors = result.output().replace("\n", "&PYEXECUTOR&").replace("\r", "&PYEXECUTOR&").split("&PYEXECUTOR&")

        self._logger.debug("Available executors found: %s" % executors)

        for item in executors:
            if item.endswith(".exe") or item.endswith(".bat"):
                self._executor = item

    """
    Set command trailer
    """
    def set_trailer(self, trailer):
        self._trailer = trailer

    """
    Run commands with commander
    """
    def _run(self, args):
        args = '%s %s' % (args, self._trailer)

        try:
            self._commander.run(self._executor, args)

            return self._commander
        except Exception as e:
            raise ExecutorException(str(e))

    """
    Run commands with pretty outputs
    """
    def run(self, args, json_output=False):
        if json_output:
            self._logger.debug('Execute with JSON output %s' % (self._executor))
            return self._run(args).json()

        self._logger.debug('Execute %s' % (self._executor))
        return self._run(args).output()

    """
    Set the execution logger
    """
    def _set_logger(self, logger):
        self._logger = Logger(logger)

class ExecutorException(Exception): ...


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyexecutor",
    version="1.0.7",
    author="WANG ZIJIAN",
    author_email="actini@outlook.com",
    description=
    """
        A light-weight command executor to run commands and return stdout and stderr.
    """,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/actini/pyexecutor",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
    ],
)
